/**
 * 
 * Last changed: $Date$ By: $Author$ URL: $HeadURL$ $Revision$
 *
 */
package de.desy.mst.libs.framework.mstapp.appinf.jaws;

import java.net.MalformedURLException;

import javax.swing.ImageIcon;

import de.desy.mst.libs.framework.mstapp.appinf.AApp;
import de.desy.mst.libs.framework.mstapp.appinf.ContactInformation;
import de.desy.mst.libs.framework.mstapp.appinf.IContactInformation;
import de.desy.mst.libs.framework.mstapp.appinf.ILib;
import de.desy.mst.libs.framework.mstapp.appinf.Lib;

/**
 * @author labudda
 *
 */
class JawsApp extends AApp {

  private final static java.util.logging.Logger log =
      java.util.logging.Logger.getLogger(JawsApp.class.getName());
  private final IContactInformation contact;
  private final String description;
  private final String title;
  private final String version;
  private final String help;
  private final String name;
  private final javax.swing.ImageIcon icon;

  JawsApp(final org.w3c.dom.Document doc) {
    this.description = JawsApp.getText(doc, "description", null);
    this.title = JawsApp.getText(doc, "title", null);
    this.version = JawsApp.getText(doc, "jnlp", "version");
    this.help = JawsApp.getText(doc, "homepage", "href");
    this.name = JawsApp.getText(doc, "title", null);
    final String vendor = JawsApp.getText(doc, "vendor", null);
    if (vendor == null) {
      this.contact = null;
    } else {
      this.contact = new ContactInformation(vendor);
    }

    final String codebase = JawsApp.getText(doc, "jnlp", "codebase");
    final String iconHref = JawsApp.getText(doc, "icon", "href");
    this.icon = JawsApp.loadIcon(codebase + iconHref);

  }

  /**
   * @param doc shall never be null
   * @param nodeName
   * @param attributeName might be null
   * @return null on node equal null node text content if attributeName is null attribute text
   *         content if neither node or attributeName is null
   */
  private static String getText(final org.w3c.dom.Document doc, final String nodeName,
      final String attributeName) {

    final org.w3c.dom.NodeList nodelist = doc.getElementsByTagName(nodeName);
    final org.w3c.dom.Node node = nodelist.item(0);
    if (node == null) {
      JawsApp.log.warning("Application have not defined node " + nodeName);
      return null;
    } else if (attributeName == null) {
      JawsApp.log
          .finer("Application have defined " + node.getTextContent() + " by node " + nodeName);
      return node.getTextContent();
    } else {
      final org.w3c.dom.Node attributeNode = node.getAttributes().getNamedItem(attributeName);
      if (attributeNode == null) {
        JawsApp.log
            .warning("Application have not defined attribute " + nodeName + " of node " + nodeName);
        return null;
      } else {
        JawsApp.log.finer("Application have defined " + attributeNode.getTextContent()
            + " by attribute " + attributeNode.getTextContent() + " of node " + nodeName);
        return attributeNode.getTextContent();
      }
    }
  }

  /**
   * @param iconPath
   * @return an icon defined by path or null
   */
  private static ImageIcon loadIcon(String iconPath) {
    java.net.URL iconUrl = null;
    try {
      iconUrl = new java.net.URL(iconPath);
      return new javax.swing.ImageIcon(iconUrl);
    } catch (MalformedURLException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
      return null;
    }
  }


  /*
   * (non-Javadoc)
   * 
   * @see de.desy.mst.libs.IApp#getContact()
   */
  @Override
  public IContactInformation getContact() {
    return this.contact;
  }

  /*
   * (non-Javadoc)
   * 
   * @see de.desy.mst.libs.IApp#getDescription()
   */
  @Override
  public String getDescription() {
    return this.description;
  }

  /*
   * (non-Javadoc)
   * 
   * @see de.desy.mst.libs.IApp#getHelp()
   */
  @Override
  public String getHelp() {
    return this.help;
  }

  /*
   * (non-Javadoc)
   * 
   * @see de.desy.mst.libs.IApp#getJars()
   */
  @Override
  public java.util.Collection<ILib> getJars() {
    final String classpath = System.getProperty("java.class.path");
    JawsApp.log.fine("Parsing java.class.path: " + classpath);
    final String[] jarNames = classpath.split(java.io.File.pathSeparator);
    final java.util.ArrayList<ILib> jars = new java.util.ArrayList<ILib>();
    for (final String jarName : jarNames) {
      if (jarName.toLowerCase().endsWith(".jar")) {
        try (final java.util.jar.JarFile jarFile = new java.util.jar.JarFile(jarName)) {
          jars.add(new Lib(jarFile));
        } catch (final java.io.IOException e) {
          JawsApp.log.log(java.util.logging.Level.WARNING, "Unable to get jar "+ jarName + " from classpath", e);
        }
      } else {
        // not a lib
      }
    }
    return jars;
  }

  /*
   * (non-Javadoc)
   * 
   * @see de.desy.mst.libs.IApp#getName()
   */
  @Override
  public String getName() {
    return this.name;
  }

  /*
   * (non-Javadoc)
   * 
   * @see de.desy.mst.libs.IApp#getVersion()
   */
  @Override
  public String getVersion() {
    return this.version;
  }

  /*
   * (non-Javadoc)
   * 
   * @see de.desy.mst.libs.IApp#getIcon()
   */
  @Override
  public javax.swing.ImageIcon getIcon() {
    return this.icon;
  }

  /*
   * (non-Javadoc)
   * 
   * @see de.desy.mst.libs.IApp#getTitle()
   */
  @Override
  public String getTitle() {
    return this.title;
  }

  /*
   * (non-Javadoc)
   * 
   * @see de.desy.mst.libs.framework.mstapp.appinf.AApp#getReleaseTimestamp()
   */
  @Override
  public String getReleaseTimestamp() {
    return (new java.util.Date()).toString();
  }

  /*
   * (non-Javadoc)
   * 
   * @see de.desy.mst.libs.framework.mstapp.appinf.AApp#isInformationValid()
   */
  @Override
  public boolean isInformationValid() {
    return this.title != null;
  }
}
